/* Automaticaly generated by genmess. Do not edit! */

const char *chkr_message[]={
/* 0 */ "Checker doesn't know this option: %s\n",
/* 1 */ "Can't find program name.\n",
/* 2 */ "My name (given by argv[0]) is %s\n",
/* 3 */ "Stop before main(). To cont, try 'fg' or 'kill -%d %d'\n",
/* 4 */ "Crt0.o has already been executed: strange loop. Abort\n",
/* 5 */ "Sorry, this ioctl is not implemented.\n",
/* 6 */ "Sorry, this fcntl is not implemented.\n",
/* 7 */ "Sorry, this syscall is not implemented.\n",
/* 8 */ "Sorry, ipc syscalls not implemented.\n",
/* 9 */ "Please, send a description of it to the author.\n",
/* 10 */ "Stack frames are:\n",
/* 11 */ "Can't find main() frame, you lose!\n",
/* 12 */ "Can't find good information in the binary file.\n",
/* 13 */ "\tpc=0x%08x in %s() at %s:%d\n",
/* 14 */ "Unknown",
/* 15 */ "<bad string table index>",
/* 16 */ "_firstblock or _lastblock is bad\n",
/* 17 */ "No blocks. Sorry.\n",
/* 18 */ "_firstblock is bad.\n",
/* 19 */ "_lastblock is bad.\n",
/* 20 */ "Bad linearity detected with blocks %u and %u.\n",
/* 21 */ "prev link of block %u is bad.\n",
/* 22 */ "Size of block 0x%x is bad\n",
/* 23 */ "The last block is not really the last one...\n",
/* 24 */ "Stop here.\n",
/* 25 */ "Block %u is an unknown aged block.\n",
/* 26 */ "Bad prev link for the younger block %u\n",
/* 27 */ "Non aged block %u in aged list.\n",
/* 28 */ "Bad number of aged block (%d instead of %d).\n",
/* 29 */ "Found too many old blocks (%d instead of %d).\n",
/* 30 */ "Bad prev link in first free block %u.\n",
/* 31 */ "Free block 0x%x has a bad state.\nAbort\n",
/* 32 */ "Block 0x%x in bad list.\n",
/* 33 */ "Bad prev link in free block %u.\n",
/* 34 */ "_heapindex is bad.\n",
/* 35 */ "Block %u is an unknown free block.\n",
/* 36 */ "Free block 0x%x is followed by another free block\n",
/* 37 */ "There is no busy block.\n",
/* 38 */ "Busy blocks are:\n",
/* 39 */ "Block (addr: 0x%x), size %u kb is busy.\n",
/* 40 */ "Block of size less than %d bytes.\n",
/* 41 */ "Block at 0x%x, size %dKb\n",
/* 42 */ "Bad next link for block %u.\n",
/* 43 */ "Found non aged block %u in aged list.\n",
/* 44 */ "There is no leak.\n",
/* 45 */ "There is 1 leak with %d potentiel leak.\n",
/* 46 */ "There are %d leaks with %d potentiel leak(s).\n",
/* 47 */ "Leaks use %u bytes (%u Kb) / %u Kb.\n",
/* 48 */ "So, %2d.%02d%% of the memory is lost.\n",
/* 49 */ "Checker found %u * %u bytes.\n",
/* 50 */ "Lost is ptr=0x%x\n",
/* 51 */ "Can't access to the symbol table.\n",
/* 52 */ "memalign is called with a null aligment. Changed to 1.\n",
/* 53 */ "No bitmap for this address.\n",
/* 54 */ "Hmmm, bit_offset is not lined up !! Ignore.\n",
/* 55 */ "Error while chkr_set_right\n",
/* 56 */ "len > bitmem.length or len == 0\n",
/* 57 */ "Error with src in chkr_copy_bm\n",
/* 58 */ "Error with dest in chkr_copy_bm\n",
/* 59 */ "Bad alignment in chkr_copy_bm\n",
/* 60 */ "Here are profile informations:\n",
/* 61 */ "The %s was handled (addressed) %d time%s.\n",
/* 62 */ "Your program handles %u times the memory.\n",
/* 63 */ "The stack limit was adjusted %u times\n",
/* 64 */ "NULL zone",
/* 65 */ "text segment",
/* 66 */ "data segment",
/* 67 */ "heap",
/* 68 */ "stack",
/* 69 */ "The block was allocated from:\n",
/* 70 */ "known stack limit= 0x%p\n",
/* 71 */ "stack bitmaped 0x%p\n",
/* 72 */ "inside a free block.\n",
/* 73 */ "The block is %u bytes length and starts at 0x%x\n",
/* 74 */ "%d bytes before the begin of the block\n",
/* 75 */ "%d bytes after the begin of the block\n",
/* 76 */ "%d bytes after the end of the block\n",
/* 77 */ "free() was called %d time(s) since this block was freed by:\n",
/* 78 */ "Memory access error\n",
/* 79 */ "When %s %d byte(s) at address 0x%p, inside the %s.\n",
/* 80 */ "You probably use a NULL pointer.\n",
/* 81 */ "THIS SHOULD PRODUCE A SEGMENTATION FAULT.\n",
/* 82 */ "You may not modify it.\n",
/* 83 */ ""
};
