#include <signal.h>
#include <stdio.h>

void show_sig(int sig)
{
 printf("Receive sig %d\n", sig);
 signal(sig, show_sig);
}

void main()
{
  int i;
  for(i=1; i < 32; i++)
    if (i != SIGTSTP)
    signal(i, show_sig);
  
  while(1)
    pause();
}
 