/* Copyright (C) 1991 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

/* The object of this file should be installed as libmcheck.a,
   so one can do -lmcheck to turn on mcheck.  */

#include <ansidecl.h>
#include "malloc.h"
#include <gnu-stabs.h>

#ifdef	HAVE_GNU_LD

static void
DEFUN_VOID (turn_on_mcheck)
{
  mcheck (NULL);
}

text_set_element (__libc_subinit, turn_on_mcheck);

#endif
