/* error list of Checker.
   Copyright 1993 Tristan Gingold
		  Written August 1993 by Tristan Gingold

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   The author may be reached (Email) at the address marc@david.saclay.cea.fr,
   or (US/French mail) as Tristan Gingold 
   			  8 rue Parmentier
   			  F91120 PALAISEAU
   			  FRANCE 
*/

#include <signal.h>
#include "checker.h"

int chkr_errno;

int chkr_show_err_flag = 1;

void chkr_perror()
{
 if(chkr_show_err_flag)
   {
     chkr_header(chkr_errlist[chkr_errno]);
     chkr_printf("\n");
   }
#ifdef CHKR_SAVESTACK
 chkr_frames_to_forget = 2;
 chkr_show_frames();
#endif  /* CHKR_SAVESTACK */
}

void chkr_abort()
{
 /* bad bad */
  chkr_do_end();
  kill(getpid(),SIGIOT);
  _exit(250);
}
