$header "const char *chkr_strings[]"

$format "%s"
M_ABORT_M1		"The -a option makes Checker aborting here. Bye.\n"
M_HELP_M1		"You can give options to Checker by the environment variable CHECKEROPTS\n"
M_HELP_M2		"Options are:\n"
M_HELP_M3		"-s --silent          Do not print the welcome message.\n"
M_HELP_M4		"-q --quiet           idem.\n"
M_HELP_M5		"-a --abort           Stop immediately.\n"
M_HELP_M6		"-h --help            Print this message.\n"
M_HELP_M7		"-n --nosymtab        Do not use symbol table.\n"
M_HELP_M8		"-o=file --output=file  Redirect the output error of Checker to 'file'.\n"
M_HELP_M9		"-i=file --image=file Set the image file (Checker find it for you)\n"
M_HELP_M10		"-p --profile         Display profile informations.\n"
M_HELP_M11		"-d=xx --disable=xxx  Disable an address.\n"
M_HELP_M12		"-S --stop            Stop just before main.\n"
M_COPY_M1		"This program has been compiled with '-checker' option.\n"
M_COPY_M2		"Checker is a memory access detector.\n"
M_COPY_M3		"Checker is distributed in the hope that it will be useful,\n"
M_COPY_M4		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
M_COPY_M5		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
M_COPY_M6		"General Public License for more details.\n"
M_COPY_M7		"For more information, set CHECKEROPTS to '-h'\n"
############# for maccess.c
M_ERROR			"ERROR"
M_2_RO_RIGHT		" R"
M_2_WO_RIGHT		" W"
M_2_RW_RIGHT		"RW"
M_IE_RIGHT_NAME		"*internal error*"
M_RO_RIGHT_NAME		"Reading"
M_WO_RIGHT_NAME		"Writing"
M_RW_RIGHT_NAME		"Modifying"
############# for vprint.c
M_FROM_CHECKER		"From Checker:\n"
