$header "const char * const chkr_errlist[]"
$declare "const char * const chkr_errlist[];"

E_UNKNOWN	"Erreur inconnue"
E_FREEFBLOCK	"Tentative de libérer un bloc libre"
E_FREEFFRAG	"Tentative de libérer un fragment libre"
E_BADADDR	"Cette adresse n'a pas été donnée par malloc"
E_NULLBSIZE	"Libération d'un bloc de taille nulle"
E_FREEBMALLOC	"free est appelé avant malloc"
E_BADRDSIZE	"La taille de la zone rouge est mauvaise"
E_CALLBRK	"brk() a été directement appelé"
E_BADBYTEPS	"'bytes_per_state' est nul. Utilisation de la valeur par défaut"
E_MEMACCERR	"Erreur d'accès à la mémoire"
E_IE_BRIGHT	"Erreur interne: Droits incorrects lors d'un contrôle"
E_IE_INSTUK	"Erreur interne: Instruction inconnue"
E_IE_BSYSCALL	"Erreur interne: Ne peut pas contrôler cet appel système"
