#include <sys/syscall.h>

void chkr_remove_symtabfile(void);

volatile void
chkr__exit(int exit_code)
{
/* really bad, but really useful */
  __asm__ volatile ("int $0x80"::"a" (SYS_exit),"b" (exit_code):"bx");
  while(1) ;
}
