/* Automaticaly generated by genmess. Do not edit! */
const char * const chkr_errlist[];
#define E_UNKNOWN	0
#define E_FREEFBLOCK	1
#define E_FREEFFRAG	2
#define E_BADADDR	3
#define E_NULLBSIZE	4
#define E_FREEBMALLOC	5
#define E_BADRDSIZE	6
#define E_CALLBRK	7
#define E_BADBYTEPS	8
#define E_MEMACCERR	9
#define E_IE_BRIGHT	10
#define E_IE_INSTUK	11
#define E_IE_BSYSCALL	12
