/* Automaticaly generated by genmess. Do not edit! */
#define M_ABORT_M1	"The -a option makes Checker aborting here. Bye.\n"
#define M_HELP_M1	"You can give options to Checker by the environment variable CHECKEROPTS\n"
#define M_HELP_M2	"Options are:\n"
#define M_HELP_M3	"-s --silent          Do not print the welcome message.\n"
#define M_HELP_M4	"-q --quiet           idem.\n"
#define M_HELP_M5	"-a --abort           Stop immediately.\n"
#define M_HELP_M6	"-h --help            Print this message.\n"
#define M_HELP_M7	"-n --nosymtab        Do not use symbol table.\n"
#define M_HELP_M8	"-o=file --output=file  Redirect the output error of Checker to 'file'.\n"
#define M_HELP_M9	"-i=file --image=file Set the image file (Checker find it for you)\n"
#define M_HELP_M10	"-p --profile         Display profile informations.\n"
#define M_HELP_M11	"-d=xx --disable=xxx  Disable an address.\n"
#define M_HELP_M12	"-S --stop            Stop just before main.\n"
#define M_COPY_M1	"This program has been compiled with '-checker' option.\n"
#define M_COPY_M2	"Checker is a memory access detector.\n"
#define M_COPY_M3	"Checker is distributed in the hope that it will be useful,\n"
#define M_COPY_M4	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
#define M_COPY_M5	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
#define M_COPY_M6	"General Public License for more details.\n"
#define M_COPY_M7	"For more information, set CHECKEROPTS to '-h'\n"
#define M_ERROR	"ERROR"
#define M_2_RO_RIGHT	" R"
#define M_2_WO_RIGHT	" W"
#define M_2_RW_RIGHT	"RW"
#define M_IE_RIGHT_NAME	"*internal error*"
#define M_RO_RIGHT_NAME	"Reading"
#define M_WO_RIGHT_NAME	"Writing"
#define M_RW_RIGHT_NAME	"Modifying"
#define M_FROM_CHECKER	"From Checker:\n"
