#include <syscall.h>
#include <sys/types.h>

static inline _syscall3 (int,read,int,fd,__ptr_t,buf,size_t,n);
static inline _syscall3 (int,write,int,fd,const __ptr_t ,buf,size_t,n);
static inline _syscall1 (int,close,int,fd);
static inline _syscall3 (long,lseek,int,fd,long,offset,int,whence);
static inline _syscall1 (int, unlink, const char*,name);
static inline _syscall0 (pid_t, getpid);
static inline _syscall2 (int,kill,pid_t,pid,int,sig);
static inline _syscall3 (int,open,const char*,filename,int,flags,int,right);
static inline _syscall2 (int,access,const char*,name,int,type);
static inline _syscall2 (int,stat,const char*,filename,struct stat*,buf);
static inline _syscall2 (int,lstat,const char*,filename,struct stat*,buf);

