/* Automaticaly generated by genmess. Do not edit! */

const char * const chkr_errlist[]={
/* 0 */ "Unknown error",
/* 1 */ "Try to free a free block",
/* 2 */ "Try to free a free fragment",
/* 3 */ "This address hadn't been returned by malloc",
/* 4 */ "Free a block which null size",
/* 5 */ "free is called before malloc",
/* 6 */ "Length of a red zone is bad",
/* 7 */ "brk() was directly called",
/* 8 */ "'bytes_per_state' is 0. Use default value",
/* 9 */ "Memory access error",
/* 10 */ "Internal error: bad right while checking",
/* 11 */ "Internal error: instruction unknown",
/* 12 */ "Internal error: can't check this syscall",
/* 13 */ ""
};
