/* DO NOT EDIT THIS FILE -- it is automagically generated.  -*- C -*- */

#if	defined(__GNU_LIBRARY__) || defined(STDC_HEADERS) || defined(USG)
#include <string.h>
#else
#define	memset(s, zero, n)	bzero ((s), (n))
#define	memcpy(d, s, n)		bcopy ((s), (d), (n))
#define memmove(d, s, n)	bcopy ((s), (d), (n))
#endif

#define _MALLOC_INTERNAL

/* The malloc headers and source files from the C library follow here.  */

