	.file	"noframe.c"
gcc2_compiled.:
___gnu_compiled_c:
.globl _tab
.text
	.align 2
_tab:
	.long 10
	.long 20
	.long 30
	.long 40
	.long 50
	.long 70
	.align 4
.globl _without_local_var
_without_local_var:
	movl 4(%esp),%edx
	addl 8(%esp),%edx
	movl %edx,%eax
	cltd
	idivl 4(%esp)
	movl %eax,%ecx
	ret
LC0:
	.ascii "%d\0"
	.align 4
.globl _with_local_var
_with_local_var:
	subl $4,%esp
	pushl %esi
	pushl %ebx
	movl 16(%esp),%ebx
	movl 20(%esp),%esi
	leal 8(%esp),%eax
	pushl %eax
	pushl $LC0
	call _scanf
	addl %esi,%ebx
	addl 16(%esp),%ebx
	movl %ebx,%eax
	addl $8,%esp
	popl %ebx
	popl %esi
	addl $4,%esp
	btl %eax, %ecx
	btw %ax, %cx
	btb %ah, %al
	ret
