/* Automaticaly generated by genmess. Do not edit! */
const char *chkr_message[];
#define M_UNKNOWN_OP	chkr_message[0]
#define M_PRGNAME_NOT_FOUND	chkr_message[1]
#define M_ARGV0_IS	chkr_message[2]
#define M_STOPPING_MYSELF	chkr_message[3]
#define M_ALREADY_INIT	chkr_message[4]
#define M_IOCTL_UNIMPLEMENT	chkr_message[5]
#define M_FCNTL_UNIMPLEMENT	chkr_message[6]
#define M_SYSCALL_UNIMPLEM	chkr_message[7]
#define M_IPC_UNIMPLEMENT	chkr_message[8]
#define M_SEND_DESCRIPTION	chkr_message[9]
#define M_STACK_FRAMES_ARE	chkr_message[10]
#define M_CANT_FIND_MAIN	chkr_message[11]
#define M_CANT_F_GOOD_INFO	chkr_message[12]
#define M_PC___IN___AT_	chkr_message[13]
#define M_UNKNOWN	chkr_message[14]
#define M_BAD_STR	chkr_message[15]
#define M_C_FB_OR_LB_BAD	chkr_message[16]
#define M_C_NO_BLOCKS	chkr_message[17]
#define M_C_FBLOCK_BAD	chkr_message[18]
#define M_C_LBLOCK_BAD	chkr_message[19]
#define M_C_BAD_LINEARITY	chkr_message[20]
#define M_C_BAD_PLINK	chkr_message[21]
#define M_C_BAD_BLOCK_SIZE	chkr_message[22]
#define M_C_BAD_LAST_BLOCK	chkr_message[23]
#define M_C_STOP_HERE	chkr_message[24]
#define M_C_U_AGED_BLOCK	chkr_message[25]
#define M_C_BAD_PL_YOUNGB	chkr_message[26]
#define M_C_NON_AGED_BLOCK	chkr_message[27]
#define M_C_BAD_NUM_AGED_B	chkr_message[28]
#define M_C_TOO_AGED_BLOCK	chkr_message[29]
#define M_C_BAD_PL_F_FREEB	chkr_message[30]
#define M_C_FREEB_BAD_STAT	chkr_message[31]
#define M_C_BLOCK_BAD_LIST	chkr_message[32]
#define M_C_BAD_PLIN_FREEB	chkr_message[33]
#define M_C_BAD_HEAPINDEX	chkr_message[34]
#define M_C_UNKNOWN_FREEB	chkr_message[35]
#define M_C_FREE_AND_FREE	chkr_message[36]
#define M_C_NO_BUSY_BLOCK	chkr_message[37]
#define M_C_BUSY_BLOCK_ARE	chkr_message[38]
#define M_C_BLOC_N_IS_BUSY	chkr_message[39]
#define M_C_BLOCK_SIZ_LESS	chkr_message[40]
#define M_C_BLOCK_AT_SIZE	chkr_message[41]
#define M_BAD_NLINK_4BLOCK	chkr_message[42]
#define M_FOUND_AGED_BLOCK	chkr_message[43]
#define M_NO_LEAK	chkr_message[44]
#define M_ONE_LEAK	chkr_message[45]
#define M_NBR_LEAKS	chkr_message[46]
#define M_LEAKS_USE_N_BYTE	chkr_message[47]
#define M_P_MEM_IS_LOST	chkr_message[48]
#define M_FOUND_N2_BYTES	chkr_message[49]
#define M_LOST_IS_PTR	chkr_message[50]
#define M_CANT_USE_SYMTAB	chkr_message[51]
#define M_NULL_ALIGN	chkr_message[52]
#define M_NO_BM_4ADDRESS	chkr_message[53]
#define M_BO_NOT_ALIGNED	chkr_message[54]
#define M_ERR_SET_RIGHT	chkr_message[55]
#define M_BAD_LEN_IN_SR	chkr_message[56]
#define M_SRC_IN_CP_BM	chkr_message[57]
#define M_DEST_IN_CP_BM	chkr_message[58]
#define M_ALIGN_IN_CP_BM	chkr_message[59]
#define M_PROF_INFO_HERE	chkr_message[60]
#define M_HANDLED_N_TIME	chkr_message[61]
#define M_HANDLED_MEM	chkr_message[62]
#define M_BM_CALLED	chkr_message[63]
#define M_NULL_ZONE	chkr_message[64]
#define M_TEXT_SEGMENT	chkr_message[65]
#define M_DATA_SEGMENT	chkr_message[66]
#define M_HEAP_SEGMENT	chkr_message[67]
#define M_STACK_SEGMENT	chkr_message[68]
#define M_BLOCK_ALLO_FRM	chkr_message[69]
#define M_KNOW_STACK_LIM	chkr_message[70]
#define M_STACK_BITMAPED	chkr_message[71]
#define M_INSIDE_FBLOCK	chkr_message[72]
#define M_BLOCK_ID	chkr_message[73]
#define M_BEFORE_BLOCK	chkr_message[74]
#define M_INSIDE_BLOCK	chkr_message[75]
#define M_AFTER_BLOCK	chkr_message[76]
#define M_N_FREE_CALLED	chkr_message[77]
#define M_MEM_ACCESS_ERR	chkr_message[78]
#define M_ACC_ERR_WHERE	chkr_message[79]
#define M_USE_NULL_PTR	chkr_message[80]
#define M_PROD_SEG_FAULT	chkr_message[81]
#define M_CANT_MODIFY_IT	chkr_message[82]
