$header "const char * const chkr_errlist[]"
$declare "const char * const chkr_errlist[];"

E_UNKNOWN	"Unknown error"
E_FREEFBLOCK	"Try to free a free block"
E_FREEFFRAG	"Try to free a free fragment"
E_BADADDR	"This address hadn't been returned by malloc"
E_NULLBSIZE	"Free a block which null size"
E_FREEBMALLOC	"free is called before malloc"
E_BADRDSIZE	"Length of a red zone is bad"
E_CALLBRK	"brk() was directly called"
E_BADBYTEPS	"'bytes_per_state' is 0. Use default value"
E_MEMACCERR	"Memory access error"
E_IE_BRIGHT	"Internal error: bad right while checking"
E_IE_INSTUK	"Internal error: instruction unknown"
E_IE_BSYSCALL	"Internal error: can't check this syscall"
