.text
.align 4
.globl __sig_jump_to
__sig_jump_to:
	pop %eax 		# /* return addr */
	pop %edx		# /* the function */
	pop %eax		# /* the new esp */
	subl $0x04, %eax	# /* adjust */
	movl %eax, %esp
	pushl %edx
	movl 28(%esp), %edi	# /* read registers from sigcontext */
	movl 32(%esp), %esi
	movl 36(%esp), %ebp
	movl 44(%esp), %ebx
	movl 48(%esp), %edx
	movl 52(%esp), %ecx
	movl 56(%esp), %eax
	ret
