/* Some systems dependant definitions
   Copyright 1993 Tristan Gingold
		  Written September 1993 Tristan Gingold

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   The author may be reached (Email) at the address marc@david.saclay.cea.fr,
   or (US/French mail) as Tristan Gingold 
   			  8 rue Parmentier
   			  F91120 PALAISEAU
   			  FRANCE
*/

#define STACK_LIMIT 0xbfffffff
#define STACK_DIRECTION -1	/* downward */
#define MIN_ADDRESS	0x50
#define PAGESIZE 0x1000
#define SIG_FRAME_SIZE 4*32	/* number of bytes for sig handler args */

#ifdef NEED_MM
#include <sys/mman.h>
#include <linux/mman.h>
#ifndef MAP_FILE
#define MAP_FILE 0
#endif
#define MM_HEAP		0x80000000	/* 256 Mb */
#define MM_STACK	0x90000000	/* 128 Mb */
#define MM_MEM		0x98000000	/* 64 Mb */
#define MM_SYM		0x9c000000	/* 64 Mb */
#define MM_PROT		PROT_READ | PROT_WRITE
#define MM_FLAGS	MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS
#define MM_FILE		(-1)
#endif /* NEED_MM */
